@@ -51,8 +51,6 @@
 #define CLUTTER_IS_ALIGN_CONSTRAINT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CLUTTER_TYPE_ALIGN_CONSTRAINT))
 #define CLUTTER_ALIGN_CONSTRAINT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), CLUTTER_TYPE_ALIGN_CONSTRAINT, ClutterAlignConstraintClass))
 
-typedef struct _ClutterAlignConstraintClass      ClutterAlignConstraintClass;
-
 struct _ClutterAlignConstraint
 {
   ClutterConstraint parent_instance;
@@ -146,7 +144,7 @@ clutter_align_constraint_update_allocation (ClutterConstraint *constraint,
       actor_height = clutter_actor_box_get_height (allocation);
       allocation->y1 = ((source_height - actor_height) * align->factor)
                      + source_y;
-      allocation->x1 = floorf (allocation->x1 + 0.5);
+      allocation->y1 = floorf (allocation->y1 + 0.5);
       allocation->y2 = allocation->y1 + actor_height;
       break;
 
