@@ -140,19 +140,23 @@ enum
   
   PROP_TIMELINE,
   PROP_ALPHA,
-  PROP_MODE
+  PROP_MODE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 static void
 timeline_new_frame_cb (ClutterTimeline *timeline,
-                       guint            current_frame_num,
+                       guint            msecs,
                        ClutterAlpha    *alpha)
 {
   ClutterAlphaPrivate *priv = alpha->priv;
 
   /* Update alpha value and notify */
   priv->alpha = clutter_alpha_get_alpha (alpha);
-  g_object_notify (G_OBJECT (alpha), "alpha");
+  _clutter_notify_by_pspec (G_OBJECT (alpha), obj_props[PROP_ALPHA]);
 }
 
 static void 
@@ -315,7 +319,6 @@ static void
 clutter_alpha_class_init (ClutterAlphaClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   object_class->set_property = clutter_alpha_set_property;
   object_class->get_property = clutter_alpha_get_property;
@@ -331,12 +334,12 @@ clutter_alpha_class_init (ClutterAlphaClass *klass)
    *
    * Since: 0.2
    */
-  pspec = g_param_spec_object ("timeline",
+  obj_props[PROP_TIMELINE] =
+    g_param_spec_object ("timeline",
                                P_("Timeline"),
                                P_("Timeline used by the alpha"),
                                CLUTTER_TYPE_TIMELINE,
                                CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (object_class, PROP_TIMELINE, pspec);
 
   /**
    * ClutterAlpha:alpha:
@@ -347,13 +350,13 @@ clutter_alpha_class_init (ClutterAlphaClass *klass)
    *
    * Since: 0.2
    */
-  pspec = g_param_spec_double ("alpha",
+  obj_props[PROP_ALPHA] =
+    g_param_spec_double ("alpha",
                                P_("Alpha value"),
                                P_("Alpha value as computed by the alpha"),
                                -1.0, 2.0,
                                0.0,
                                CLUTTER_PARAM_READABLE);
-  g_object_class_install_property (object_class, PROP_ALPHA, pspec);
 
   /**
    * ClutterAlpha:mode:
@@ -368,13 +371,17 @@ clutter_alpha_class_init (ClutterAlphaClass *klass)
    *
    * Since: 1.0
    */
-  pspec = g_param_spec_ulong ("mode",
+  obj_props[PROP_MODE] =
+    g_param_spec_ulong ("mode",
                               P_("Mode"),
                               P_("Progress mode"),
                               0, G_MAXULONG,
                               CLUTTER_CUSTOM_MODE,
                               G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (object_class, PROP_MODE, pspec);
+
+  _clutter_object_class_install_properties (object_class,
+                                            PROP_LAST,
+                                            obj_props);
 }
 
 static void
@@ -488,7 +495,7 @@ clutter_alpha_set_closure (ClutterAlpha *alpha,
   clutter_alpha_set_closure_internal (alpha, closure);
 
   priv->mode = CLUTTER_CUSTOM_MODE;
-  g_object_notify (G_OBJECT (alpha), "mode");
+  _clutter_notify_by_pspec (G_OBJECT (alpha), obj_props[PROP_MODE]);
 }
 
 /**
@@ -524,7 +531,7 @@ clutter_alpha_set_func (ClutterAlpha    *alpha,
   clutter_alpha_set_closure_internal (alpha, closure);
 
   priv->mode = CLUTTER_CUSTOM_MODE;
-  g_object_notify (G_OBJECT (alpha), "mode");
+  _clutter_notify_by_pspec (G_OBJECT (alpha), obj_props[PROP_MODE]);
 }
 
 /**
@@ -569,7 +576,7 @@ clutter_alpha_set_timeline (ClutterAlpha    *alpha,
                         alpha);
     }
 
-  g_object_notify (G_OBJECT (alpha), "timeline");
+  _clutter_notify_by_pspec (G_OBJECT (alpha), obj_props[PROP_TIMELINE]);
 }
 
 /**
@@ -1291,7 +1298,7 @@ clutter_alpha_set_mode (ClutterAlpha *alpha,
   else
     g_assert_not_reached ();
 
-  g_object_notify (G_OBJECT (alpha), "mode");
+  _clutter_notify_by_pspec (G_OBJECT (alpha), obj_props[PROP_MODE]);
 }
 
 static gulong
@@ -1306,7 +1313,7 @@ register_alpha_internal (AlphaData *alpha_data)
 }
 
 /**
- * clutter_alpha_register_func:
+ * clutter_alpha_register_func: (skip)
  * @func: a #ClutterAlphaFunc
  * @data: user data to pass to @func, or %NULL
  *
@@ -1346,6 +1353,8 @@ clutter_alpha_register_func (ClutterAlphaFunc func,
  *
  * The logical id is always greater than %CLUTTER_ANIMATION_LAST.
  *
+ * Rename to: clutter_alpha_register_func
+ *
  * Return value: the logical id of the alpha function
  *
  * Since: 1.0
