@@ -44,12 +44,13 @@
 
 #include "clutter-backend-private.h"
 #include "clutter-debug.h"
-#include "clutter-fixed.h"
+#include "clutter-event-private.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-profile.h"
 #include "clutter-stage-manager-private.h"
 #include "clutter-stage-private.h"
+#include "clutter-version.h"
 
 #include <cogl/cogl.h>
 
@@ -68,6 +69,8 @@ struct _ClutterBackendPrivate
 
   gfloat units_per_em;
   gint32 units_serial;
+
+  GList *event_translators;
 };
 
 enum
@@ -84,17 +87,16 @@ static guint backend_signals[LAST_SIGNAL] = { 0, };
 static void
 clutter_backend_dispose (GObject *gobject)
 {
-  ClutterMainContext *clutter_context;
+  ClutterBackendPrivate *priv = CLUTTER_BACKEND (gobject)->priv;
 
-  clutter_context = _clutter_context_get_default ();
+  /* clear the events still in the queue of the main context */
+  _clutter_clear_events_queue ();
 
-  if (clutter_context && clutter_context->events_queue)
+  /* remove all event translators */
+  if (priv->event_translators != NULL)
     {
-      g_queue_foreach (clutter_context->events_queue,
-                       (GFunc) clutter_event_free,
-                       NULL);
-      g_queue_free (clutter_context->events_queue);
-      clutter_context->events_queue = NULL;
+      g_list_free (priv->event_translators);
+      priv->event_translators = NULL;
     }
 
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
@@ -179,11 +181,10 @@ clutter_backend_real_resolution_changed (ClutterBackend *backend)
   ClutterSettings *settings;
   gint dpi;
 
-  context = _clutter_context_get_default ();
-
   settings = clutter_settings_get_default ();
   g_object_get (settings, "font-dpi", &dpi, NULL);
 
+  context = _clutter_context_get_default ();
   if (context->font_map != NULL)
     cogl_pango_font_map_set_resolution (context->font_map, dpi / 1024.0);
 
@@ -204,6 +205,51 @@ clutter_backend_real_font_changed (ClutterBackend *backend)
   CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
+static void
+clutter_backend_real_redraw (ClutterBackend *backend,
+                             ClutterStage   *stage)
+{
+  ClutterStageWindow *impl;
+
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage))
+    return;
+
+  impl = _clutter_stage_get_window (stage);
+  if (impl == NULL)
+    return;
+
+  _clutter_stage_window_redraw (impl);
+}
+
+static gboolean
+clutter_backend_real_translate_event (ClutterBackend *backend,
+                                      gpointer        native,
+                                      ClutterEvent   *event)
+{
+  ClutterBackendPrivate *priv = backend->priv;
+  GList *l;
+
+  for (l = priv->event_translators;
+       l != NULL;
+       l = l->next)
+    {
+      ClutterEventTranslator *translator = l->data;
+      ClutterTranslateReturn retval;
+
+      retval = _clutter_event_translator_translate_event (translator,
+                                                          native,
+                                                          event);
+
+      if (retval == CLUTTER_TRANSLATE_QUEUE)
+        return TRUE;
+
+      if (retval == CLUTTER_TRANSLATE_REMOVE)
+        return FALSE;
+    }
+
+  return FALSE;
+}
+
 static void
 clutter_backend_class_init (ClutterBackendClass *klass)
 {
@@ -243,6 +289,8 @@ clutter_backend_class_init (ClutterBackendClass *klass)
 
   klass->resolution_changed = clutter_backend_real_resolution_changed;
   klass->font_changed = clutter_backend_real_font_changed;
+  klass->translate_event = clutter_backend_real_translate_event;
+  klass->redraw = clutter_backend_real_redraw;
 }
 
 static void
@@ -262,7 +310,7 @@ _clutter_backend_add_options (ClutterBackend *backend,
 {
   ClutterBackendClass *klass;
 
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  g_assert (CLUTTER_IS_BACKEND (backend));
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->add_options)
@@ -275,7 +323,7 @@ _clutter_backend_pre_parse (ClutterBackend  *backend,
 {
   ClutterBackendClass *klass;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
+  g_assert (CLUTTER_IS_BACKEND (backend));
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->pre_parse)
@@ -290,7 +338,7 @@ _clutter_backend_post_parse (ClutterBackend  *backend,
 {
   ClutterBackendClass *klass;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
+  g_assert (CLUTTER_IS_BACKEND (backend));
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->post_parse)
@@ -308,8 +356,8 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
   ClutterStageManager *stage_manager;
   ClutterStageWindow *stage_window;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
-  g_return_val_if_fail (CLUTTER_IS_STAGE (wrapper), NULL);
+  g_assert (CLUTTER_IS_BACKEND (backend));
+  g_assert (CLUTTER_IS_STAGE (wrapper));
 
   stage_manager = clutter_stage_manager_get_default ();
 
@@ -333,7 +381,6 @@ void
 _clutter_backend_redraw (ClutterBackend *backend,
                          ClutterStage   *stage)
 {
-  ClutterBackendClass *klass;
   CLUTTER_STATIC_COUNTER (redraw_counter,
                           "_clutter_backend_redraw counter",
                           "Increments for each _clutter_backend_redraw call",
@@ -347,9 +394,7 @@ _clutter_backend_redraw (ClutterBackend *backend,
   CLUTTER_COUNTER_INC (_clutter_uprof_context, redraw_counter);
   CLUTTER_TIMER_START (_clutter_uprof_context, redraw_timer);
 
-  klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  if (G_LIKELY (klass->redraw))
-    klass->redraw (backend, stage);
+  CLUTTER_BACKEND_GET_CLASS (backend)->redraw (backend, stage);
 
   CLUTTER_TIMER_STOP (_clutter_uprof_context, redraw_timer);
 }
@@ -360,8 +405,6 @@ _clutter_backend_create_context (ClutterBackend  *backend,
 {
   ClutterBackendClass *klass;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
-
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->create_context)
     return klass->create_context (backend, error);
@@ -384,8 +427,8 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
 {
   static ClutterStage *current_context_stage = NULL;
 
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_assert (CLUTTER_IS_BACKEND (backend));
+  g_assert (CLUTTER_IS_STAGE (stage));
 
   if (current_context_stage != stage || !CLUTTER_ACTOR_IS_REALIZED (stage))
     {
@@ -454,7 +497,7 @@ _clutter_backend_get_features (ClutterBackend *backend)
   ClutterBackendClass *klass;
   GError *error;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
+  g_assert (CLUTTER_IS_BACKEND (backend));
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
 
@@ -463,7 +506,7 @@ _clutter_backend_get_features (ClutterBackend *backend)
    * context already exists this should be a no-op
    */
   error = NULL;
-  if (klass->create_context)
+  if (klass->create_context != NULL)
     {
       gboolean res;
 
@@ -492,14 +535,8 @@ void
 _clutter_backend_init_events (ClutterBackend *backend)
 {
   ClutterBackendClass *klass;
-  ClutterMainContext  *clutter_context;
 
-  clutter_context = _clutter_context_get_default ();
-
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-  g_return_if_fail (clutter_context != NULL);
-
-  clutter_context->events_queue = g_queue_new ();
+  g_assert (CLUTTER_IS_BACKEND (backend));
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->init_events)
@@ -512,8 +549,6 @@ _clutter_backend_get_units_per_em (ClutterBackend       *backend,
 {
   ClutterBackendPrivate *priv;
 
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
-
   priv = backend->priv;
 
   /* recompute for the font description, but do not cache the result */
@@ -533,10 +568,6 @@ _clutter_backend_copy_event_data (ClutterBackend     *backend,
 {
   ClutterBackendClass *klass;
 
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-  g_return_if_fail (src != NULL);
-  g_return_if_fail (dest != NULL);
-
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->copy_event_data != NULL)
     klass->copy_event_data (backend, src, dest);
@@ -548,9 +579,6 @@ _clutter_backend_free_event_data (ClutterBackend *backend,
 {
   ClutterBackendClass *klass;
 
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-  g_return_if_fail (event != NULL);
-
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->free_event_data != NULL)
     klass->free_event_data (backend, event);
@@ -578,8 +606,6 @@ clutter_get_default_backend (void)
   return clutter_context->backend;
 }
 
-/* FIXME: below should probably be moved into clutter_main */
-
 /**
  * clutter_backend_set_double_click_time:
  * @backend: a #ClutterBackend
@@ -877,8 +903,6 @@ clutter_backend_get_font_name (ClutterBackend *backend)
 gint32
 _clutter_backend_get_units_serial (ClutterBackend *backend)
 {
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
-
   return backend->priv->units_serial;
 }
 
@@ -887,9 +911,33 @@ _clutter_backend_translate_event (ClutterBackend *backend,
                                   gpointer        native,
                                   ClutterEvent   *event)
 {
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
-
   return CLUTTER_BACKEND_GET_CLASS (backend)->translate_event (backend,
                                                                native,
                                                                event);
 }
+
+void
+_clutter_backend_add_event_translator (ClutterBackend         *backend,
+                                       ClutterEventTranslator *translator)
+{
+  ClutterBackendPrivate *priv = backend->priv;
+
+  if (g_list_find (priv->event_translators, translator) != NULL)
+    return;
+
+  priv->event_translators =
+    g_list_prepend (priv->event_translators, translator);
+}
+
+void
+_clutter_backend_remove_event_translator (ClutterBackend         *backend,
+                                          ClutterEventTranslator *translator)
+{
+  ClutterBackendPrivate *priv = backend->priv;
+
+  if (g_list_find (priv->event_translators, translator) == NULL)
+    return;
+
+  priv->event_translators =
+    g_list_remove (priv->event_translators, translator);
+}
