@@ -23,7 +23,9 @@
 
 /**
  * SECTION:clutter-behaviour
+ * @Title: ClutterBehaviour
  * @short_description: Class for providing behaviours to actors
+ * @Deprecated: 1.6: Use the implicit animation API instead.
  *
  * #ClutterBehaviour is the base class for implementing behaviours.  A
  * behaviour is a controller object for #ClutterActor<!-- -->s; you can
@@ -62,15 +64,15 @@
  * instance bound to the behaviour and apply it to the desiderd property
  * (or properties) of every actor controlled by the behaviour.
  *
- * #ClutterBehaviour is available since Clutter 0.2
+ * #ClutterBehaviour is available since Clutter 0.2.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "clutter-actor.h"
 #include "clutter-behaviour.h"
+
 #include "clutter-debug.h"
 #include "clutter-main.h"
 #include "clutter-marshal.h"
@@ -141,21 +143,9 @@ clutter_knot_equal (const ClutterKnot *knot_a,
   return knot_a->x == knot_b->x && knot_a->y == knot_b->y;
 }
 
-GType
-clutter_knot_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (G_UNLIKELY (!our_type))
-    {
-      our_type =
-        g_boxed_type_register_static (I_("ClutterKnot"),
-                                      (GBoxedCopyFunc) clutter_knot_copy,
-                                      (GBoxedFreeFunc) clutter_knot_free);
-    }
-
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (ClutterKnot, clutter_knot,
+                     clutter_knot_copy,
+                     clutter_knot_free);
 
 static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
 
@@ -288,7 +278,6 @@ static void
 clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   object_class->dispose      = clutter_behaviour_dispose;
   object_class->set_property = clutter_behaviour_set_property;
@@ -304,13 +293,16 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
    *
    * Since: 0.2
    */
-  pspec = g_param_spec_object ("alpha",
-                               P_("Alpha"),
-                               P_("Alpha Object to drive the behaviour"),
-                               CLUTTER_TYPE_ALPHA,
-                               CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_ALPHA] = pspec;
-  g_object_class_install_property (object_class, PROP_ALPHA, pspec);
+  obj_props[PROP_ALPHA] =
+    g_param_spec_object ("alpha",
+                         P_("Alpha"),
+                         P_("Alpha Object to drive the behaviour"),
+                         CLUTTER_TYPE_ALPHA,
+                         CLUTTER_PARAM_READWRITE);
+
+  _clutter_object_class_install_properties (object_class,
+                                            PROP_LAST,
+                                            obj_props);
 
   klass->alpha_notify = clutter_behaviour_alpha_notify_unimplemented;
 
@@ -511,7 +503,7 @@ clutter_behaviour_get_nth_actor (ClutterBehaviour *behave,
 /**
  * clutter_behaviour_actors_foreach:
  * @behave: a #ClutterBehaviour
- * @func: a function called for each actor
+ * @func: (scope call): a function called for each actor
  * @data: optional data to be passed to the function, or %NULL
  *
  * Calls @func for every actor driven by @behave.
@@ -655,7 +647,7 @@ clutter_behaviour_set_alpha (ClutterBehaviour *behave,
  * for derived classes to use this in there alpha notify method but use 
  * #clutter_behaviour_actors_foreach as it avoids alot of needless allocations.
  *
- * Return value: (transfer container) (element-type ClutterActor): a list of
+ * Return value: (transfer container) (element-type Clutter.Actor): a list of
  *   actors. You should free the returned list with g_slist_free() when
  *   finished using it.
  * 
