@@ -87,6 +87,8 @@ struct _ClutterOffscreenEffectPrivate
 
   gfloat target_width;
   gfloat target_height;
+
+  gint old_opacity_override;
 };
 
 G_DEFINE_ABSTRACT_TYPE (ClutterOffscreenEffect,
@@ -148,7 +150,17 @@ update_fbo (ClutterEffect *effect, int fbo_width, int fbo_height)
     return TRUE;
 
   if (priv->target == COGL_INVALID_HANDLE)
-    priv->target = cogl_material_new ();
+    {
+      priv->target = cogl_material_new ();
+
+      /* We're always going to render the texture at a 1:1 texel:pixel
+         ratio so we can use 'nearest' filtering to decrease the
+         effects of rounding errors in the geometry calculation */
+      cogl_material_set_layer_filters (priv->target,
+                                       0, /* layer_index */
+                                       COGL_MATERIAL_FILTER_NEAREST,
+                                       COGL_MATERIAL_FILTER_NEAREST);
+    }
 
   texture =
     clutter_offscreen_effect_create_texture (self, fbo_width, fbo_height);
@@ -209,13 +221,18 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
    * we need to redirect its rendering offscreen and its position will
    * be used to setup an offset viewport */
   if (clutter_actor_get_paint_box (priv->actor, &box))
-    clutter_actor_box_get_size (&box, &fbo_width, &fbo_height);
+    {
+      clutter_actor_box_get_size (&box, &fbo_width, &fbo_height);
+      clutter_actor_box_get_origin (&box, &priv->x_offset, &priv->y_offset);
+    }
   else
     {
       /* If we can't get a valid paint box then we fallback to
        * creating a full stage size fbo. */
       ClutterActor *stage = _clutter_actor_get_stage_internal (priv->actor);
       clutter_actor_get_size (stage, &fbo_width, &fbo_height);
+      priv->x_offset = 0.0f;
+      priv->y_offset = 0.0f;
     }
 
   /* First assert that the framebuffer is the right size... */
@@ -234,7 +251,6 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
    * but offset it so that the actor of interest lands on our
    * framebuffer. */
   clutter_actor_get_size (priv->stage, &width, &height);
-  clutter_actor_box_get_origin (&box, &priv->x_offset, &priv->y_offset);
 
   /* Expand the viewport if the actor is partially off-stage,
    * otherwise the actor will end up clipped to the stage viewport
@@ -296,6 +312,8 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
    * texture with the actor's paint opacity, so we need to do this to avoid
    * multiplying the opacity twice.
    */
+  priv->old_opacity_override =
+    _clutter_actor_get_opacity_override (priv->actor);
   _clutter_actor_set_opacity_override (priv->actor, 0xff);
 
   return TRUE;
@@ -354,8 +372,8 @@ clutter_offscreen_effect_post_paint (ClutterEffect *effect)
   cogl_matrix_translate (&modelview, priv->x_offset, priv->y_offset, 0.0f);
   cogl_set_modelview_matrix (&modelview);
 
-  /* Remove the opacity override */
-  _clutter_actor_set_opacity_override (priv->actor, -1);
+  /* Restore the previous opacity override */
+  _clutter_actor_set_opacity_override (priv->actor, priv->old_opacity_override);
 
   /* paint the target material; this is virtualized for
    * sub-classes that require special hand-holding
@@ -475,3 +493,22 @@ clutter_offscreen_effect_create_texture (ClutterOffscreenEffect *effect,
                                                                       width,
                                                                       height);
 }
+
+gboolean
+_clutter_offscreen_effect_get_target_size (ClutterOffscreenEffect *effect,
+                                           gfloat                 *width,
+                                           gfloat                 *height)
+{
+  ClutterOffscreenEffectPrivate *priv = effect->priv;
+
+  if (priv->target == NULL)
+    return FALSE;
+
+  if (width)
+    *width = priv->target_width;
+
+  if (height)
+    *height = priv->target_height;
+
+  return TRUE;
+}
