@@ -24,15 +24,14 @@
 
 /**
  * SECTION:clutter-shader-effect
- * @short_description: Base class for fragment shader effects
+ * @short_description: Base class for shader effects
  * @See_Also: #ClutterEffect, #ClutterOffscreenEffect
  *
  * #ClutterShaderEffect is an abstract class that implements all the
- * plumbing for creating #ClutterEffect<!-- -->s using GLSL fragment
- * shaders.
+ * plumbing for creating #ClutterEffect<!-- -->s using GLSL shaders.
  *
  * #ClutterShaderEffect creates an offscreen buffer and then applies the GLSL
- * fragment shader (after checking whether the compilation and linking were
+ * shader (after checking whether the compilation and linking were
  * successfull) to the buffer before painting it on screen.
  *
  * <refsect2 id="ClutterShaderEffect-implementing">
@@ -144,29 +143,32 @@ enum
 {
   PROP_0,
 
-  PROP_SHADER_TYPE
+  PROP_SHADER_TYPE,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_ABSTRACT_TYPE (ClutterShaderEffect,
                         clutter_shader_effect,
                         CLUTTER_TYPE_OFFSCREEN_EFFECT);
 
 static inline void
-clutter_shader_effect_clear (ClutterShaderEffect *effect,
+clutter_shader_effect_clear (ClutterShaderEffect *self,
                              gboolean             reset_uniforms)
 {
-  ClutterShaderEffectPrivate *priv = effect->priv;
+  ClutterShaderEffectPrivate *priv = self->priv;
 
-  if (priv->shader != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->shader);
-      priv->shader = COGL_INVALID_HANDLE;
-    }
+  if (priv->shader != COGL_INVALID_HANDLE && !priv->is_compiled)
+    cogl_handle_unref (priv->shader);
 
   if (priv->program != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->program);
+
       priv->program = COGL_INVALID_HANDLE;
+      priv->shader = COGL_INVALID_HANDLE;
     }
 
   if (reset_uniforms && priv->uniforms != NULL)
@@ -180,26 +182,6 @@ clutter_shader_effect_clear (ClutterShaderEffect *effect,
   priv->source_set = FALSE;
 }
 
-static void
-clutter_shader_effect_reset_uniforms (ClutterShaderEffect *effect)
-{
-  ClutterShaderEffectPrivate *priv = effect->priv;
-  GHashTableIter iter;
-  gpointer key, value;
-
-  if (priv->uniforms == NULL)
-    return;
-
-  key = value = NULL;
-  g_hash_table_iter_init (&iter, priv->uniforms);
-  while (g_hash_table_iter_next (&iter, &key, &value))
-    {
-      ShaderUniform *uniform = value;
-
-      uniform->location = -1;
-    }
-}
-
 static void
 clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
 {
@@ -208,6 +190,9 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
   gpointer key, value;
   gsize size;
 
+  if (priv->program == COGL_INVALID_HANDLE)
+    return;
+
   if (priv->uniforms == NULL)
     return;
 
@@ -226,33 +211,53 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
           const GLfloat *floats;
 
           floats = clutter_value_get_shader_float (&uniform->value, &size);
-          cogl_program_uniform_float (uniform->location, size, 1, floats);
+          cogl_program_set_uniform_float (priv->program, uniform->location,
+                                          size, 1,
+                                          floats);
         }
       else if (CLUTTER_VALUE_HOLDS_SHADER_INT (&uniform->value))
         {
           const GLint *ints;
 
           ints = clutter_value_get_shader_int (&uniform->value, &size);
-          cogl_program_uniform_int (uniform->location, size, 1, ints);
+          cogl_program_set_uniform_int (priv->program, uniform->location,
+                                        size, 1,
+                                        ints);
         }
       else if (CLUTTER_VALUE_HOLDS_SHADER_MATRIX (&uniform->value))
         {
           const GLfloat *matrix;
 
           matrix = clutter_value_get_shader_matrix (&uniform->value, &size);
-          cogl_program_uniform_matrix (uniform->location, size, 1, FALSE, matrix);
+          cogl_program_set_uniform_matrix (priv->program, uniform->location,
+                                           size, 1,
+                                           FALSE,
+                                           matrix);
         }
       else if (G_VALUE_HOLDS_FLOAT (&uniform->value))
         {
           const GLfloat float_val = g_value_get_float (&uniform->value);
 
-          cogl_program_uniform_float (uniform->location, 1, 1, &float_val);
+          cogl_program_set_uniform_float (priv->program, uniform->location,
+                                          1, 1,
+                                          &float_val);
+        }
+      else if (G_VALUE_HOLDS_DOUBLE (&uniform->value))
+        {
+          const GLfloat float_val =
+            (GLfloat) g_value_get_double (&uniform->value);
+
+          cogl_program_set_uniform_float (priv->program, uniform->location,
+                                          1, 1,
+                                          &float_val);
         }
       else if (G_VALUE_HOLDS_INT (&uniform->value))
         {
           const GLint int_val = g_value_get_int (&uniform->value);
 
-          cogl_program_uniform_int (uniform->location, 1, 1, &int_val);
+          cogl_program_set_uniform_int (priv->program, uniform->location,
+                                        1, 1,
+                                        &int_val);
         }
       else
         g_warning ("Invalid uniform of type '%s' for name '%s'",
@@ -281,9 +286,6 @@ clutter_shader_effect_set_actor (ClutterActorMeta *meta,
       return;
     }
 
-  clutter_shader_effect_clear (self, FALSE);
-  clutter_shader_effect_reset_uniforms (self);
-
   parent = CLUTTER_ACTOR_META_CLASS (clutter_shader_effect_parent_class);
   parent->set_actor (meta, actor);
 
@@ -294,25 +296,6 @@ clutter_shader_effect_set_actor (ClutterActorMeta *meta,
 
   CLUTTER_NOTE (SHADER, "Preparing shader effect of type '%s'",
                 G_OBJECT_TYPE_NAME (meta));
-
-  priv->program = cogl_create_program ();
-
-  switch (priv->shader_type)
-    {
-    case CLUTTER_FRAGMENT_SHADER:
-      priv->shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
-      break;
-
-    case CLUTTER_VERTEX_SHADER:
-      priv->shader = cogl_create_shader (COGL_SHADER_TYPE_VERTEX);
-      break;
-
-    default:
-      priv->shader = COGL_INVALID_HANDLE;
-      break;
-    }
-
-  g_assert (priv->shader != COGL_INVALID_HANDLE);
 }
 
 static void
@@ -320,16 +303,17 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
 {
   ClutterShaderEffectPrivate *priv = CLUTTER_SHADER_EFFECT (effect)->priv;
   ClutterOffscreenEffectClass *parent;
+  CoglHandle material;
 
   /* we haven't been prepared or we don't have support for
    * GLSL shaders in Clutter
    */
   if (priv->program == COGL_INVALID_HANDLE ||
       priv->shader == COGL_INVALID_HANDLE)
-    return;
+    goto out;
 
   if (!priv->source_set)
-    return;
+    goto out;
 
   if (!priv->is_compiled)
     {
@@ -349,10 +333,12 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
           cogl_handle_unref (priv->program);
           priv->shader = COGL_INVALID_HANDLE;
 
-          return;
+          goto out;
         }
 
       cogl_program_attach_shader (priv->program, priv->shader);
+      cogl_handle_unref (priv->shader);
+
       cogl_program_link (priv->program);
 
       priv->is_compiled = TRUE;
@@ -361,17 +347,17 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
   CLUTTER_NOTE (SHADER, "Applying the shader effect of type '%s'",
                 G_OBJECT_TYPE_NAME (effect));
 
-  /* set the shader */
-  cogl_program_use (priv->program);
-
   clutter_shader_effect_update_uniforms (CLUTTER_SHADER_EFFECT (effect));
 
+  /* associate the program to the offscreen target material */
+  material = clutter_offscreen_effect_get_target (effect);
+  cogl_material_set_user_program (material, priv->program);
+
+out:
   /* paint the offscreen buffer */
   parent = CLUTTER_OFFSCREEN_EFFECT_CLASS (clutter_shader_effect_parent_class);
   parent->paint_target (effect);
 
-  /* unset the shader */
-  cogl_program_use (COGL_INVALID_HANDLE);
 }
 
 static void
@@ -410,15 +396,11 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterOffscreenEffectClass *offscreen_class;
-  GParamSpec *pspec;
 
   offscreen_class = CLUTTER_OFFSCREEN_EFFECT_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (ClutterShaderEffectPrivate));
 
-  gobject_class->set_property = clutter_shader_effect_set_property;
-  gobject_class->finalize = clutter_shader_effect_finalize;
-
   /**
    * ClutterShaderEffect:shader-type:
    *
@@ -428,14 +410,19 @@ clutter_shader_effect_class_init (ClutterShaderEffectClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_enum ("shader-type",
-                             P_("Shader Type"),
-                             P_("The type of shader used"),
-                             CLUTTER_TYPE_SHADER_TYPE,
-                             CLUTTER_FRAGMENT_SHADER,
-                             CLUTTER_PARAM_WRITABLE |
-                             G_PARAM_CONSTRUCT_ONLY);
-  g_object_class_install_property (gobject_class, PROP_SHADER_TYPE, pspec);
+  obj_props[PROP_SHADER_TYPE] =
+    g_param_spec_enum ("shader-type",
+                       P_("Shader Type"),
+                       P_("The type of shader used"),
+                       CLUTTER_TYPE_SHADER_TYPE,
+                       CLUTTER_FRAGMENT_SHADER,
+                       CLUTTER_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
+  gobject_class->set_property = clutter_shader_effect_set_property;
+  gobject_class->finalize = clutter_shader_effect_finalize;
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 
   meta_class->set_actor = clutter_shader_effect_set_actor;
 
@@ -510,11 +497,10 @@ shader_uniform_new (const gchar  *name,
 {
   ShaderUniform *retval;
 
-  retval = g_slice_new (ShaderUniform);
+  retval = g_slice_new0 (ShaderUniform);
   retval->name = g_strdup (name);
   retval->type = G_VALUE_TYPE (value);
   retval->location = -1;
-  retval->value = (GValue){0};
 
   g_value_init (&retval->value, retval->type);
   g_value_copy (value, &retval->value);
@@ -577,7 +563,8 @@ clutter_shader_effect_add_uniform (ClutterShaderEffect *effect,
  * %CLUTTER_TYPE_SHADER_INT, for an array of integer values;
  * %CLUTTER_TYPE_SHADER_FLOAT, for an array of floating point values;
  * and %CLUTTER_TYPE_SHADER_MATRIX, for a matrix of floating point
- * values
+ * values. It also accepts %G_TYPE_DOUBLE for compatibility with other
+ * languages than C.
  *
  * Since: 1.4
  */
@@ -586,14 +573,10 @@ clutter_shader_effect_set_uniform_value (ClutterShaderEffect *effect,
                                          const gchar         *name,
                                          const GValue        *value)
 {
-  ClutterShaderEffectPrivate *priv;
-
   g_return_if_fail (CLUTTER_IS_SHADER_EFFECT (effect));
   g_return_if_fail (name != NULL);
   g_return_if_fail (value != NULL);
 
-  priv = effect->priv;
-
   clutter_shader_effect_add_uniform (effect, name, value);
 }
 
@@ -825,8 +808,28 @@ clutter_shader_effect_set_shader_source (ClutterShaderEffect *effect,
   if (priv->source_set)
     return TRUE;
 
+  if (priv->program == COGL_INVALID_HANDLE)
+    priv->program = cogl_create_program ();
+
   if (priv->shader == COGL_INVALID_HANDLE)
-    return FALSE;
+    {
+      switch (priv->shader_type)
+        {
+        case CLUTTER_FRAGMENT_SHADER:
+          priv->shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
+          break;
+
+        case CLUTTER_VERTEX_SHADER:
+          priv->shader = cogl_create_shader (COGL_SHADER_TYPE_VERTEX);
+          break;
+
+        default:
+          priv->shader = COGL_INVALID_HANDLE;
+          break;
+        }
+    }
+
+  g_assert (priv->shader != COGL_INVALID_HANDLE);
 
   cogl_shader_source (priv->shader, source);
 
