@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2010 Intel Corporation.
+ * Copyright (C) 2010,2011 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,167 +37,14 @@
 #include "cogl-texture-driver.h"
 #include "cogl-pipeline-opengl-private.h"
 #include "cogl-debug.h"
+#include "cogl-framebuffer-private.h"
+#include "cogl-blit.h"
 
 #include <stdlib.h>
 
-#ifdef HAVE_COGL_GLES2
+static void _cogl_atlas_free (CoglAtlas *atlas);
 
-#include "../gles/cogl-gles2-wrapper.h"
-
-#else /* HAVE_COGL_GLES2 */
-
-#define glGenFramebuffers                 ctx->drv.pf_glGenFramebuffers
-#define glBindFramebuffer                 ctx->drv.pf_glBindFramebuffer
-#define glFramebufferTexture2D            ctx->drv.pf_glFramebufferTexture2D
-#define glCheckFramebufferStatus          ctx->drv.pf_glCheckFramebufferStatus
-#define glDeleteFramebuffers              ctx->drv.pf_glDeleteFramebuffers
-
-#endif /* HAVE_COGL_GLES2 */
-
-#ifndef GL_FRAMEBUFFER
-#define GL_FRAMEBUFFER          0x8D40
-#endif
-#ifndef GL_FRAMEBUFFER_BINDING
-#define GL_FRAMEBUFFER_BINDING  0x8CA6
-#endif
-#ifndef GL_COLOR_ATTACHMENT0
-#define GL_COLOR_ATTACHMENT0    0x8CE0
-#endif
-#ifndef GL_FRAMEBUFFER_COMPLETE
-#define GL_FRAMEBUFFER_COMPLETE 0x8CD5
-#endif
-
-/* If we want to do mulitple blits from a texture (such as when
-   reorganizing the atlas) then it's quicker to download all of the
-   data once and upload multiple times from that. This struct is used
-   to keep the image data for a series of blits */
-
-typedef struct _CoglAtlasBlitData
-{
-  CoglHandle src_tex, dst_tex;
-
-  /* If we're using an FBO to blit, then FBO will be non-zero and
-     old_fbo will be the previous framebuffer binding */
-  GLuint fbo, old_fbo;
-
-  /* If we're not using an FBO then we g_malloc a buffer and copy the
-     complete texture data in */
-  unsigned char *image_data;
-  CoglPixelFormat format;
-  int bpp;
-  unsigned int src_height, src_width;
-
-  GLenum dst_gl_target;
-} CoglAtlasBlitData;
-
-static void
-_cogl_atlas_blit_begin (CoglAtlasBlitData *data,
-                        CoglHandle dst_tex,
-                        CoglHandle src_tex)
-{
-  GLenum src_gl_target;
-  GLuint src_gl_texture;
-  GLuint dst_gl_texture;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  data->dst_tex = dst_tex;
-  data->src_tex = src_tex;
-  data->fbo = 0;
-
-  /* If we can use an FBO then we don't need to download the data and
-     we can tell GL to blit directly between the textures */
-  if (cogl_features_available (COGL_FEATURE_OFFSCREEN) &&
-      !cogl_texture_is_sliced (dst_tex) &&
-      cogl_texture_get_gl_texture (src_tex, &src_gl_texture, &src_gl_target) &&
-      cogl_texture_get_gl_texture (dst_tex, &dst_gl_texture,
-                                   &data->dst_gl_target))
-    {
-      /* Ideally we would use the cogl-offscreen API here, but I'd
-         rather avoid creating a stencil renderbuffer which you can't
-         currently do */
-      /* Preserve the previous framebuffer binding so we don't trample
-         on cogl-offscreen */
-      data->old_fbo = 0;
-      GE( glGetIntegerv (GL_FRAMEBUFFER_BINDING, (GLint *) &data->old_fbo) );
-
-      _cogl_texture_set_filters (src_tex, GL_NEAREST, GL_NEAREST);
-
-      /* Create an FBO to read from the src texture */
-      GE( glGenFramebuffers (1, &data->fbo) );
-      GE( glBindFramebuffer (GL_FRAMEBUFFER, data->fbo) );
-      GE( glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                                  src_gl_target, src_gl_texture, 0) );
-      if (glCheckFramebufferStatus (GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
-        {
-          /* The FBO failed for whatever reason so we'll fallback to
-             reading the texture data */
-          GE( glBindFramebuffer (GL_FRAMEBUFFER, data->old_fbo) );
-          GE( glDeleteFramebuffers (1, &data->fbo) );
-          data->fbo = 0;
-        }
-
-      _cogl_bind_gl_texture_transient (data->dst_gl_target,
-                                       dst_gl_texture,
-                                       FALSE);
-    }
-
-  if (data->fbo)
-    COGL_NOTE (ATLAS, "Blit set up using an FBO");
-  else
-    {
-      /* We need to retrieve the entire texture data (there is no
-         glGetTexSubImage2D) */
-
-      data->format = cogl_texture_get_format (src_tex);
-      data->bpp = _cogl_get_format_bpp (data->format);
-      data->src_width = cogl_texture_get_width (src_tex);
-      data->src_height = cogl_texture_get_height (src_tex);
-
-      data->image_data = g_malloc (data->bpp * data->src_width *
-                                   data->src_height);
-      cogl_texture_get_data (src_tex, data->format,
-                             data->src_width * data->bpp, data->image_data);
-    }
-}
-
-static void
-_cogl_atlas_blit (CoglAtlasBlitData *data,
-                  unsigned int src_x,
-                  unsigned int src_y,
-                  unsigned int dst_x,
-                  unsigned int dst_y,
-                  unsigned int width,
-                  unsigned int height)
-{
-  /* If we have an FBO then we can do a fast blit */
-  if (data->fbo)
-    GE( glCopyTexSubImage2D (data->dst_gl_target, 0, dst_x, dst_y, src_x, src_y,
-                             width, height) );
-  else
-    cogl_texture_set_region (data->dst_tex,
-                             src_x, src_y,
-                             dst_x, dst_y,
-                             width, height,
-                             data->src_width, data->src_height,
-                             data->format,
-                             data->src_width * data->bpp,
-                             data->image_data);
-}
-
-static void
-_cogl_atlas_blit_end (CoglAtlasBlitData *data)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  if (data->fbo)
-    {
-      GE( glBindFramebuffer (GL_FRAMEBUFFER, data->old_fbo) );
-      GE( glDeleteFramebuffers (1, &data->fbo) );
-    }
-  else
-    g_free (data->image_data);
-}
+COGL_OBJECT_INTERNAL_DEFINE (Atlas, atlas);
 
 CoglAtlas *
 _cogl_atlas_new (CoglPixelFormat texture_format,
@@ -211,20 +58,24 @@ _cogl_atlas_new (CoglPixelFormat texture_format,
   atlas->texture = NULL;
   atlas->flags = flags;
   atlas->texture_format = texture_format;
-  _cogl_callback_list_init (&atlas->reorganize_callbacks);
+  _cogl_callback_list_init (&atlas->pre_reorganize_callbacks);
+  _cogl_callback_list_init (&atlas->post_reorganize_callbacks);
 
-  return atlas;
+  return _cogl_atlas_object_new (atlas);
 }
 
-void
+static void
 _cogl_atlas_free (CoglAtlas *atlas)
 {
+  COGL_NOTE (ATLAS, "%p: Atlas destroyed", atlas);
+
   if (atlas->texture)
     cogl_handle_unref (atlas->texture);
   if (atlas->map)
     _cogl_rectangle_map_free (atlas->map);
 
-  _cogl_callback_list_destroy (&atlas->reorganize_callbacks);
+  _cogl_callback_list_destroy (&atlas->pre_reorganize_callbacks);
+  _cogl_callback_list_destroy (&atlas->post_reorganize_callbacks);
 
   g_free (atlas);
 }
@@ -247,7 +98,7 @@ _cogl_atlas_migrate (CoglAtlas               *atlas,
                      void                    *skip_user_data)
 {
   unsigned int i;
-  CoglAtlasBlitData blit_data;
+  CoglBlitData blit_data;
 
   /* If the 'disable migrate' flag is set then we won't actually copy
      the textures to their new location. Instead we'll just invoke the
@@ -260,20 +111,20 @@ _cogl_atlas_migrate (CoglAtlas               *atlas,
                                  &textures[i].new_position);
   else
     {
-      _cogl_atlas_blit_begin (&blit_data, new_texture, old_texture);
+      _cogl_blit_begin (&blit_data, new_texture, old_texture);
 
       for (i = 0; i < n_textures; i++)
         {
           /* Skip the texture that is being added because it doesn't contain
              any data yet */
           if (textures[i].user_data != skip_user_data)
-            _cogl_atlas_blit (&blit_data,
-                              textures[i].old_position.x,
-                              textures[i].old_position.y,
-                              textures[i].new_position.x,
-                              textures[i].new_position.y,
-                              textures[i].new_position.width,
-                              textures[i].new_position.height);
+            _cogl_blit (&blit_data,
+                        textures[i].old_position.x,
+                        textures[i].old_position.y,
+                        textures[i].new_position.x,
+                        textures[i].new_position.y,
+                        textures[i].new_position.width,
+                        textures[i].new_position.height);
 
           /* Update the texture position */
           atlas->update_position_cb (textures[i].user_data,
@@ -281,7 +132,7 @@ _cogl_atlas_migrate (CoglAtlas               *atlas,
                                      &textures[i].new_position);
         }
 
-      _cogl_atlas_blit_end (&blit_data);
+      _cogl_blit_end (&blit_data);
     }
 }
 
@@ -459,9 +310,15 @@ _cogl_atlas_compare_size_cb (const void *a,
 }
 
 static void
-_cogl_atlas_notify_reorganize (CoglAtlas *atlas)
+_cogl_atlas_notify_pre_reorganize (CoglAtlas *atlas)
 {
-  _cogl_callback_list_invoke (&atlas->reorganize_callbacks);
+  _cogl_callback_list_invoke (&atlas->pre_reorganize_callbacks);
+}
+
+static void
+_cogl_atlas_notify_post_reorganize (CoglAtlas *atlas)
+{
+  _cogl_callback_list_invoke (&atlas->post_reorganize_callbacks);
 }
 
 gboolean
@@ -483,15 +340,15 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
                                user_data,
                                &new_position))
     {
-      int waste =
-        _cogl_rectangle_map_get_remaining_space (atlas->map) *
-        100 / (_cogl_rectangle_map_get_width (atlas->map) *
-               _cogl_rectangle_map_get_height (atlas->map));
-      COGL_NOTE (ATLAS, "Atlas is %ix%i, has %i textures and is %i%% waste",
+      COGL_NOTE (ATLAS, "%p: Atlas is %ix%i, has %i textures and is %i%% waste",
+                 atlas,
                  _cogl_rectangle_map_get_width (atlas->map),
                  _cogl_rectangle_map_get_height (atlas->map),
                  _cogl_rectangle_map_get_n_rectangles (atlas->map),
-                 waste);
+                 /* waste as a percentage */
+                 _cogl_rectangle_map_get_remaining_space (atlas->map) *
+                 100 / (_cogl_rectangle_map_get_width (atlas->map) *
+                        _cogl_rectangle_map_get_height (atlas->map)));
 
       atlas->update_position_cb (user_data,
                                  atlas->texture,
@@ -500,8 +357,13 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
       return TRUE;
     }
 
-  /* We need to reorganise the atlas so we'll get an array of all the
-     textures currently in the atlas. */
+  /* If we make it here then we need to reorganize the atlas. First
+     we'll notify any users of the atlas that this is going to happen
+     so that for example in CoglAtlasTexture it can notify that the
+     storage has changed and cause a flush */
+  _cogl_atlas_notify_pre_reorganize (atlas);
+
+  /* Get an array of all the textures currently in the atlas. */
   data.n_textures = 0;
   if (atlas->map == NULL)
     data.textures = g_malloc (sizeof (CoglAtlasRepositionData));
@@ -557,7 +419,7 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
   /* If we can't create a map with the texture then give up */
   if (new_map == NULL)
     {
-      COGL_NOTE (ATLAS, "Could not fit texture in the atlas");
+      COGL_NOTE (ATLAS, "%p: Could not fit texture in the atlas", atlas);
       ret = FALSE;
     }
   /* We need to migrate the existing textures into a new texture */
@@ -566,7 +428,7 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
              _cogl_rectangle_map_get_width (new_map),
              _cogl_rectangle_map_get_height (new_map))) == COGL_INVALID_HANDLE)
     {
-      COGL_NOTE (ATLAS, "Could not create a CoglTexture2D");
+      COGL_NOTE (ATLAS, "%p: Could not create a CoglTexture2D", atlas);
       _cogl_rectangle_map_free (new_map);
       ret = FALSE;
     }
@@ -574,10 +436,9 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
     {
       int waste;
 
-      _cogl_atlas_notify_reorganize (atlas);
-
       COGL_NOTE (ATLAS,
-                 "Atlas %s with size %ix%i",
+                 "%p: Atlas %s with size %ix%i",
+                 atlas,
                  atlas->map == NULL ||
                  _cogl_rectangle_map_get_width (atlas->map) !=
                  _cogl_rectangle_map_get_width (new_map) ||
@@ -614,7 +475,8 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
                100 / (_cogl_rectangle_map_get_width (atlas->map) *
                       _cogl_rectangle_map_get_height (atlas->map)));
 
-      COGL_NOTE (ATLAS, "Atlas is %ix%i, has %i textures and is %i%% waste",
+      COGL_NOTE (ATLAS, "%p: Atlas is %ix%i, has %i textures and is %i%% waste",
+                 atlas,
                  _cogl_rectangle_map_get_width (atlas->map),
                  _cogl_rectangle_map_get_height (atlas->map),
                  _cogl_rectangle_map_get_n_rectangles (atlas->map),
@@ -625,6 +487,8 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
 
   g_free (data.textures);
 
+  _cogl_atlas_notify_post_reorganize (atlas);
+
   return ret;
 }
 
@@ -634,10 +498,12 @@ _cogl_atlas_remove (CoglAtlas *atlas,
 {
   _cogl_rectangle_map_remove (atlas->map, rectangle);
 
-  COGL_NOTE (ATLAS, "Removed rectangle sized %ix%i",
+  COGL_NOTE (ATLAS, "%p: Removed rectangle sized %ix%i",
+             atlas,
              rectangle->width,
              rectangle->height);
-  COGL_NOTE (ATLAS, "Atlas is %ix%i, has %i textures and is %i%% waste",
+  COGL_NOTE (ATLAS, "%p: Atlas is %ix%i, has %i textures and is %i%% waste",
+             atlas,
              _cogl_rectangle_map_get_width (atlas->map),
              _cogl_rectangle_map_get_height (atlas->map),
              _cogl_rectangle_map_get_n_rectangles (atlas->map),
@@ -656,7 +522,7 @@ _cogl_atlas_copy_rectangle (CoglAtlas        *atlas,
                             CoglPixelFormat   format)
 {
   CoglHandle tex;
-  CoglAtlasBlitData blit_data;
+  CoglBlitData blit_data;
 
   /* Create a new texture at the right size */
   tex = cogl_texture_new_with_size (width, height, flags, format);
@@ -664,29 +530,42 @@ _cogl_atlas_copy_rectangle (CoglAtlas        *atlas,
   /* Blit the data out of the atlas to the new texture. If FBOs
      aren't available this will end up having to copy the entire
      atlas texture */
-  _cogl_atlas_blit_begin (&blit_data, tex, atlas->texture);
-  _cogl_atlas_blit (&blit_data,
+  _cogl_blit_begin (&blit_data, tex, atlas->texture);
+  _cogl_blit (&blit_data,
                     x, y,
                     0, 0,
                     width, height);
-  _cogl_atlas_blit_end (&blit_data);
+  _cogl_blit_end (&blit_data);
 
   return tex;
 }
 
 void
 _cogl_atlas_add_reorganize_callback (CoglAtlas            *atlas,
-                                     CoglCallbackListFunc  callback,
+                                     CoglCallbackListFunc  pre_callback,
+                                     CoglCallbackListFunc  post_callback,
                                      void                 *user_data)
 {
-  _cogl_callback_list_add (&atlas->reorganize_callbacks, callback, user_data);
+  if (pre_callback)
+    _cogl_callback_list_add (&atlas->pre_reorganize_callbacks,
+                             pre_callback,
+                             user_data);
+  if (post_callback)
+    _cogl_callback_list_add (&atlas->post_reorganize_callbacks,
+                             post_callback,
+                             user_data);
 }
 
 void
 _cogl_atlas_remove_reorganize_callback (CoglAtlas            *atlas,
-                                        CoglCallbackListFunc  callback,
+                                        CoglCallbackListFunc  pre_callback,
+                                        CoglCallbackListFunc  post_callback,
                                         void                 *user_data)
 {
-  _cogl_callback_list_remove (&atlas->reorganize_callbacks,
-                              callback, user_data);
+  if (pre_callback)
+    _cogl_callback_list_remove (&atlas->pre_reorganize_callbacks,
+                                pre_callback, user_data);
+  if (post_callback)
+    _cogl_callback_list_remove (&atlas->post_reorganize_callbacks,
+                                post_callback, user_data);
 }
