@@ -51,7 +51,7 @@ cogl_clip_push_window_rectangle (int x_offset,
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = _cogl_get_framebuffer ();
+  framebuffer = _cogl_get_draw_buffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   clip_state->stacks->data =
@@ -82,7 +82,7 @@ cogl_clip_push_rectangle (float x_1,
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = _cogl_get_framebuffer ();
+  framebuffer = _cogl_get_draw_buffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   cogl_get_modelview_matrix (&modelview_matrix);
@@ -115,7 +115,7 @@ cogl_clip_push_from_path_preserve (void)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = _cogl_get_framebuffer ();
+  framebuffer = _cogl_get_draw_buffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   cogl_get_modelview_matrix (&modelview_matrix);
@@ -151,7 +151,7 @@ cogl_clip_pop (void)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = _cogl_get_framebuffer ();
+  framebuffer = _cogl_get_draw_buffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   _cogl_clip_pop_real (clip_state);
@@ -169,14 +169,15 @@ _cogl_clip_state_flush (CoglClipState *clip_state)
 void
 cogl_clip_ensure (void)
 {
+  CoglFramebuffer *framebuffer = _cogl_get_draw_buffer ();
   CoglClipState *clip_state;
 
-  clip_state = _cogl_framebuffer_get_clip_state (_cogl_get_framebuffer ());
+  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
   /* Flushing the clip state doesn't cause the journal to be
      flushed. This function may be being called by an external
      application however so it makes sense to flush the journal
      here */
-  _cogl_journal_flush ();
+  _cogl_framebuffer_flush_journal (framebuffer);
   _cogl_clip_state_flush (clip_state);
 }
 
@@ -194,7 +195,7 @@ cogl_clip_stack_save (void)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = _cogl_get_framebuffer ();
+  framebuffer = _cogl_get_draw_buffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   _cogl_clip_stack_save_real (clip_state);
@@ -224,7 +225,7 @@ cogl_clip_stack_restore (void)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = _cogl_get_framebuffer ();
+  framebuffer = _cogl_get_draw_buffer ();
   clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
   _cogl_clip_stack_restore_real (clip_state);
@@ -250,30 +251,15 @@ _cogl_clip_state_destroy (CoglClipState *clip_state)
 }
 
 CoglClipStack *
-_cogl_get_clip_stack (void)
+_cogl_clip_state_get_stack (CoglClipState *clip_state)
 {
-  CoglFramebuffer *framebuffer;
-  CoglClipState *clip_state;
-
-  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
-
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
   return clip_state->stacks->data;
 }
 
 void
-_cogl_set_clip_stack (CoglClipStack *stack)
+_cogl_clip_state_set_stack (CoglClipState *clip_state,
+                            CoglClipStack *stack)
 {
-  CoglFramebuffer *framebuffer;
-  CoglClipState *clip_state;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
   /* Replace the top of the stack of stacks */
   _cogl_clip_stack_ref (stack);
   _cogl_clip_stack_unref (clip_state->stacks->data);
