@@ -62,19 +62,19 @@ static gboolean gl_is_indirect = FALSE;
 static void
 _cogl_init_feature_overrides (CoglContext *ctx)
 {
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_VBOS)))
     ctx->feature_flags &= ~COGL_FEATURE_VBOS;
 
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_PBOS))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_PBOS)))
     ctx->feature_flags &= ~COGL_FEATURE_PBOS;
 
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_ARBFP))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_ARBFP)))
     ctx->feature_flags &= ~COGL_FEATURE_SHADERS_ARBFP;
 
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_GLSL)))
     ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
 
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_NPOT_TEXTURES))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_NPOT_TEXTURES)))
     ctx->feature_flags &= ~(COGL_FEATURE_TEXTURE_NPOT |
                             COGL_FEATURE_TEXTURE_NPOT_BASIC |
                             COGL_FEATURE_TEXTURE_NPOT_MIPMAP |
@@ -166,10 +166,10 @@ cogl_create_context (void)
   _context->default_gl_texture_2d_tex = COGL_INVALID_HANDLE;
   _context->default_gl_texture_rect_tex = COGL_INVALID_HANDLE;
 
-  _context->journal = g_array_new (FALSE, FALSE, sizeof (CoglJournalEntry));
-  _context->logged_vertices = g_array_new (FALSE, FALSE, sizeof (GLfloat));
+  _context->framebuffers = NULL;
+
   _context->journal_flush_attributes_array =
-    g_array_new (TRUE, FALSE, sizeof (CoglVertexAttribute *));
+    g_array_new (TRUE, FALSE, sizeof (CoglAttribute *));
   _context->journal_clip_bounds = NULL;
 
   _context->polygon_vertices = g_array_new (FALSE, FALSE, sizeof (float));
@@ -244,6 +244,7 @@ cogl_create_context (void)
   _context->rectangle_short_indices_len = 0;
 
   _context->texture_download_pipeline = COGL_INVALID_HANDLE;
+  _context->blit_texture_pipeline = COGL_INVALID_HANDLE;
 
 #ifndef HAVE_COGL_GLES2
   /* The default for GL_ALPHA_TEST is to always pass which is equivalent to
@@ -287,14 +288,20 @@ cogl_create_context (void)
 
   _context->atlases = NULL;
 
+  _context->buffer_map_fallback_array = g_byte_array_new ();
+  _context->buffer_map_fallback_in_use = FALSE;
+
   /* As far as I can tell, GL_POINT_SPRITE doesn't have any effect
      unless GL_COORD_REPLACE is enabled for an individual
      layer. Therefore it seems like it should be ok to just leave it
      enabled all the time instead of having to have a set property on
      each pipeline to track whether any layers have point sprite
-     coords enabled */
+     coords enabled. We don't need to do this for GLES2 because point
+     sprites are handled using a builtin varying in the shader. */
+#ifndef HAVE_COGL_GLES2
   if (cogl_features_available (COGL_FEATURE_POINT_SPRITE))
     GE (glEnable (GL_POINT_SPRITE));
+#endif
 
   return TRUE;
 }
@@ -327,10 +334,9 @@ _cogl_destroy_context (void)
   if (_context->texture_pipeline)
     cogl_handle_unref (_context->texture_pipeline);
 
-  if (_context->journal)
-    g_array_free (_context->journal, TRUE);
-  if (_context->logged_vertices)
-    g_array_free (_context->logged_vertices, TRUE);
+  if (_context->blit_texture_pipeline)
+    cogl_handle_unref (_context->blit_texture_pipeline);
+
   if (_context->journal_flush_attributes_array)
     g_array_free (_context->journal_flush_attributes_array, TRUE);
   if (_context->journal_clip_bounds)
@@ -382,6 +388,8 @@ _cogl_destroy_context (void)
   g_hash_table_unref (_context->arbfp_cache);
 #endif
 
+  g_byte_array_free (_context->buffer_map_fallback_array, TRUE);
+
   g_free (_context);
 }
 
