@@ -168,4 +168,14 @@ OPT (DISABLE_BLENDING,
      "disable-program-caches",
      "Disable program caches",
      "Disable fallback caches for arbfp and glsl programs")
-
+OPT (DISABLE_FAST_READ_PIXEL,
+     "Root Cause",
+     "disable-fast-read-pixel",
+     "Disable read pixel optimization",
+     "Disable optimization for reading 1px for simple "
+     "scenes of opaque rectangles")
+OPT (CLIPPING,
+     "Cogl Tracing",
+     "clipping",
+     "Trace clipping",
+     "Logs information about how Cogl is implementing clipping")
