@@ -35,7 +35,8 @@
 #include "cogl-context.h"
 #include "cogl-handle.h"
 #include "cogl-journal-private.h"
-#include "cogl-material-opengl-private.h"
+#include "cogl-pipeline-private.h"
+#include "cogl-pipeline-opengl-private.h"
 
 #include <string.h>
 #include <math.h>
@@ -100,8 +101,6 @@ _cogl_texture_3d_manual_repeat_cb (const float *coords,
                                 slice_coords + 1, slice_coords + 3);
 
   data->callback (COGL_TEXTURE (data->tex_3d),
-                  data->tex_3d->gl_texture,
-                  GL_TEXTURE_3D,
                   slice_coords,
                   coords,
                   data->user_data);
@@ -179,7 +178,7 @@ _cogl_texture_3d_create_base (unsigned int     width,
   CoglTexture3D *tex_3d = g_new (CoglTexture3D, 1);
   CoglTexture *tex = COGL_TEXTURE (tex_3d);
 
-  tex->vtable = &cogl_texture_3d_vtable;
+  _cogl_texture_init (tex, &cogl_texture_3d_vtable);
 
   tex_3d->width = width;
   tex_3d->height = height;
@@ -572,6 +571,7 @@ _cogl_texture_3d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
          GL_GENERATE_MIPMAP and reuploading the first pixel */
       if (cogl_features_available (COGL_FEATURE_OFFSCREEN))
         _cogl_texture_driver_gl_generate_mipmaps (GL_TEXTURE_3D);
+#ifndef HAVE_COGL_GLES2
       else
         {
           GE( glTexParameteri (GL_TEXTURE_3D,
@@ -592,6 +592,7 @@ _cogl_texture_3d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
                                GL_GENERATE_MIPMAP,
                                GL_FALSE) );
         }
+#endif
 
       tex_3d->mipmaps_dirty = FALSE;
     }
