@@ -97,28 +97,6 @@ cogl_vertex_buffer_new (unsigned int n_vertices);
 unsigned int
 cogl_vertex_buffer_get_n_vertices (CoglHandle handle);
 
-/**
- * CoglAttributeType:
- * @COGL_ATTRIBUTE_TYPE_BYTE: Data is the same size of a byte
- * @COGL_ATTRIBUTE_TYPE_UNSIGNED_BYTE: Data is the same size of an
- *   unsigned byte
- * @COGL_ATTRIBUTE_TYPE_SHORT: Data is the same size of a short integer
- * @COGL_ATTRIBUTE_TYPE_UNSIGNED_SHORT: Data is the same size of
- *   an unsigned short integer
- * @COGL_ATTRIBUTE_TYPE_FLOAT: Data is the same size of a float
- *
- * Data types for the components of cogl_vertex_buffer_add()
- *
- * Since: 1.0
- */
-typedef enum {
-  COGL_ATTRIBUTE_TYPE_BYTE = GL_BYTE,
-  COGL_ATTRIBUTE_TYPE_UNSIGNED_BYTE = GL_UNSIGNED_BYTE,
-  COGL_ATTRIBUTE_TYPE_SHORT = GL_SHORT,
-  COGL_ATTRIBUTE_TYPE_UNSIGNED_SHORT = GL_UNSIGNED_SHORT,
-  COGL_ATTRIBUTE_TYPE_FLOAT = GL_FLOAT
-} CoglAttributeType;
-
 /**
  * cogl_vertex_buffer_add:
  * @handle: A vertex buffer handle
@@ -151,7 +129,8 @@ typedef enum {
  *   must remain valid until you either call cogl_vertex_buffer_submit() or
  *   issue a draw call.
  *
- * Adds an attribute to a buffer.
+ * Adds an attribute to a buffer, or replaces a previously added
+ * attribute with the same name.
  *
  * You either can use one of the built-in names such as "gl_Vertex", or
  * "gl_MultiTexCoord0" to add standard attributes, like positions, colors
@@ -253,31 +232,6 @@ void
 cogl_vertex_buffer_enable (CoglHandle  handle,
 			   const char *attribute_name);
 
-/**
- * CoglVerticesMode:
- * @COGL_VERTICES_MODE_POINTS: FIXME, equivalent to %GL_POINTS
- * @COGL_VERTICES_MODE_LINE_STRIP: FIXME, equivalent to %GL_LINE_STRIP
- * @COGL_VERTICES_MODE_LINE_LOOP: FIXME, equivalent to %GL_LINE_LOOP
- * @COGL_VERTICES_MODE_LINES: FIXME, equivalent to %GL_LINES
- * @COGL_VERTICES_MODE_TRIANGLE_STRIP: FIXME, equivalent to %GL_TRIANGLE_STRIP
- * @COGL_VERTICES_MODE_TRIANGLE_FAN: FIXME, equivalent to %GL_TRIANGLE_FAN
- * @COGL_VERTICES_MODE_TRIANGLES: FIXME, equivalent to %GL_TRIANGLES
- *
- * How vertices passed to cogl_vertex_buffer_draw() and
- * cogl_vertex_buffer_draw_elements() should be interpreted
- *
- * Since: 1.0
- */
-typedef enum {
-  COGL_VERTICES_MODE_POINTS = GL_POINTS,
-  COGL_VERTICES_MODE_LINE_STRIP = GL_LINE_STRIP,
-  COGL_VERTICES_MODE_LINE_LOOP = GL_LINE_LOOP,
-  COGL_VERTICES_MODE_LINES = GL_LINES,
-  COGL_VERTICES_MODE_TRIANGLE_STRIP = GL_TRIANGLE_STRIP,
-  COGL_VERTICES_MODE_TRIANGLE_FAN = GL_TRIANGLE_FAN,
-  COGL_VERTICES_MODE_TRIANGLES = GL_TRIANGLES
-} CoglVerticesMode;
-
 /**
  * cogl_vertex_buffer_draw:
  * @handle: A vertex buffer handle
@@ -298,28 +252,6 @@ cogl_vertex_buffer_draw (CoglHandle       handle,
 		         int              first,
 		         int              count);
 
-/**
- * CoglIndicesType:
- * @COGL_INDICES_TYPE_UNSIGNED_BYTE: Your indices are unsigned bytes
- * @COGL_INDICES_TYPE_UNSIGNED_SHORT: Your indices are unsigned shorts
- * @COGL_INDICES_TYPE_UNSIGNED_INT: Your indices are unsigned ints
- *
- * You should aim to use the smallest data type that gives you enough
- * range, since it reduces the size of your index array and can help
- * reduce the demand on memory bandwidth.
- *
- * Note that %COGL_INDICES_TYPE_UNSIGNED_INT is only supported if the
- * %COGL_FEATURE_UNSIGNED_INT_INDICES feature is available. This
- * should always be available on OpenGL but on OpenGL ES it will only
- * be available if the GL_OES_element_index_uint extension is
- * advertized.
- */
-typedef enum {
-  COGL_INDICES_TYPE_UNSIGNED_BYTE,
-  COGL_INDICES_TYPE_UNSIGNED_SHORT,
-  COGL_INDICES_TYPE_UNSIGNED_INT,
-} CoglIndicesType;
-
 /**
  * cogl_vertex_buffer_indices_new:
  * @indices_type: a #CoglIndicesType specifying the data type used for
@@ -343,7 +275,7 @@ cogl_vertex_buffer_indices_new (CoglIndicesType  indices_type,
 
 /**
  * cogl_vertex_buffer_indices_get_type:
- * @handle: An indices handle
+ * @indices: An indices handle
  *
  * Queries back the data type used for the given indices
  *
