@@ -269,85 +269,17 @@ clutter_stage_egl_add_redraw_clip (ClutterStageWindow *stage_window,
 }
 
 static void
-clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
+clutter_stage_egl_redraw (ClutterStageWindow *stage_window)
 {
+  ClutterStageEGL *stage_egl = CLUTTER_STAGE_EGL (stage_window);
+  ClutterBackend *backend = clutter_get_default_backend ();
+  ClutterBackendEGL *backend_egl = CLUTTER_BACKEND_EGL (backend);
+  ClutterActor *wrapper;
+  EGLSurface egl_surface;
+  gboolean may_use_clipped_redraw;
+  gboolean use_clipped_redraw;
 #ifdef COGL_HAS_X11_SUPPORT
-  clutter_stage_egl_parent_iface = g_type_interface_peek_parent (iface);
-
-  iface->realize = clutter_stage_egl_realize;
-  iface->unrealize = clutter_stage_egl_unrealize;
-
-  /* the rest is inherited from ClutterStageX11 */
-
-#else /* COGL_HAS_X11_SUPPORT */
-
-  iface->realize = clutter_stage_egl_realize;
-  iface->unrealize = clutter_stage_egl_unrealize;
-  iface->set_fullscreen = clutter_stage_egl_set_fullscreen;
-  iface->set_title = clutter_stage_egl_set_title;
-  iface->set_cursor_visible = clutter_stage_egl_set_cursor_visible;
-  iface->get_wrapper = clutter_stage_egl_get_wrapper;
-  iface->get_geometry = clutter_stage_egl_get_geometry;
-  iface->resize = clutter_stage_egl_resize;
-  iface->show = clutter_stage_egl_show;
-  iface->hide = clutter_stage_egl_hide;
-
-#endif /* COGL_HAS_X11_SUPPORT */
-
-  iface->add_redraw_clip = clutter_stage_egl_add_redraw_clip;
-  iface->has_redraw_clips = clutter_stage_egl_has_redraw_clips;
-  iface->ignoring_redraw_clips = clutter_stage_egl_ignoring_redraw_clips;
-}
-
-#ifdef COGL_HAS_X11_SUPPORT
-static void
-clutter_stage_egl_dispose (GObject *gobject)
-{
-  G_OBJECT_CLASS (_clutter_stage_egl_parent_class)->dispose (gobject);
-}
-
-static void
-_clutter_stage_egl_class_init (ClutterStageEGLClass *klass)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-
-  gobject_class->dispose = clutter_stage_egl_dispose;
-}
-
-static void
-_clutter_stage_egl_init (ClutterStageEGL *stage)
-{
-  stage->egl_surface = EGL_NO_SURFACE;
-}
-
-#else /* COGL_HAS_X11_SUPPORT */
-
-static void
-_clutter_stage_egl_class_init (ClutterStageEGLClass *klass)
-{
-}
-
-static void
-_clutter_stage_egl_init (ClutterStageEGL *stage)
-{
-  /* Without X we only support one surface and that is associated
-   * with the backend directly instead of the stage */
-}
-
-#endif /* COGL_HAS_X11_SUPPORT */
-
-void
-_clutter_stage_egl_redraw (ClutterStageEGL *stage_egl,
-                           ClutterStage    *stage)
-{
-  ClutterBackend     *backend = clutter_get_default_backend ();
-  ClutterBackendEGL  *backend_egl = CLUTTER_BACKEND_EGL (backend);
-  ClutterActor       *wrapper;
-  EGLSurface          egl_surface;
-  gboolean            may_use_clipped_redraw;
-  gboolean            use_clipped_redraw;
-#ifdef COGL_HAS_X11_SUPPORT
-  ClutterStageX11    *stage_x11 = CLUTTER_STAGE_X11 (stage_egl);
+  ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_egl);
 
   wrapper = CLUTTER_ACTOR (stage_x11->wrapper);
   egl_surface = stage_egl->egl_surface;
@@ -388,11 +320,12 @@ _clutter_stage_egl_redraw (ClutterStageEGL *stage_egl,
                                        stage_egl->bounding_redraw_clip.y,
                                        stage_egl->bounding_redraw_clip.width,
                                        stage_egl->bounding_redraw_clip.height);
-      _clutter_stage_do_paint (stage, &stage_egl->bounding_redraw_clip);
+      _clutter_stage_do_paint (CLUTTER_STAGE (wrapper),
+                               &stage_egl->bounding_redraw_clip);
       cogl_clip_pop ();
     }
   else
-    _clutter_stage_do_paint (stage, NULL);
+    _clutter_stage_do_paint (CLUTTER_STAGE (wrapper), NULL);
 
   if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS &&
       may_use_clipped_redraw)
@@ -430,8 +363,7 @@ _clutter_stage_egl_redraw (ClutterStageEGL *stage_egl,
 
       cogl_push_matrix ();
       cogl_matrix_init_identity (&modelview);
-      _clutter_actor_apply_modelview_transform (CLUTTER_ACTOR (stage),
-                                                &modelview);
+      _clutter_actor_apply_modelview_transform (wrapper, &modelview);
       cogl_set_modelview_matrix (&modelview);
       cogl_set_source (outline);
       cogl_vertex_buffer_draw (vbo, COGL_VERTICES_MODE_LINE_LOOP,
@@ -489,3 +421,72 @@ _clutter_stage_egl_redraw (ClutterStageEGL *stage_egl,
 
   stage_egl->frame_count++;
 }
+
+static void
+clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
+{
+#ifdef COGL_HAS_X11_SUPPORT
+  clutter_stage_egl_parent_iface = g_type_interface_peek_parent (iface);
+
+  iface->realize = clutter_stage_egl_realize;
+  iface->unrealize = clutter_stage_egl_unrealize;
+
+  /* the rest is inherited from ClutterStageX11 */
+
+#else /* COGL_HAS_X11_SUPPORT */
+
+  iface->realize = clutter_stage_egl_realize;
+  iface->unrealize = clutter_stage_egl_unrealize;
+  iface->set_fullscreen = clutter_stage_egl_set_fullscreen;
+  iface->set_title = clutter_stage_egl_set_title;
+  iface->set_cursor_visible = clutter_stage_egl_set_cursor_visible;
+  iface->get_wrapper = clutter_stage_egl_get_wrapper;
+  iface->get_geometry = clutter_stage_egl_get_geometry;
+  iface->resize = clutter_stage_egl_resize;
+  iface->show = clutter_stage_egl_show;
+  iface->hide = clutter_stage_egl_hide;
+
+#endif /* COGL_HAS_X11_SUPPORT */
+
+  iface->add_redraw_clip = clutter_stage_egl_add_redraw_clip;
+  iface->has_redraw_clips = clutter_stage_egl_has_redraw_clips;
+  iface->ignoring_redraw_clips = clutter_stage_egl_ignoring_redraw_clips;
+  iface->redraw = clutter_stage_egl_redraw;
+}
+
+#ifdef COGL_HAS_X11_SUPPORT
+static void
+clutter_stage_egl_dispose (GObject *gobject)
+{
+  G_OBJECT_CLASS (_clutter_stage_egl_parent_class)->dispose (gobject);
+}
+
+static void
+_clutter_stage_egl_class_init (ClutterStageEGLClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+
+  gobject_class->dispose = clutter_stage_egl_dispose;
+}
+
+static void
+_clutter_stage_egl_init (ClutterStageEGL *stage)
+{
+  stage->egl_surface = EGL_NO_SURFACE;
+}
+
+#else /* COGL_HAS_X11_SUPPORT */
+
+static void
+_clutter_stage_egl_class_init (ClutterStageEGLClass *klass)
+{
+}
+
+static void
+_clutter_stage_egl_init (ClutterStageEGL *stage)
+{
+  /* Without X we only support one surface and that is associated
+   * with the backend directly instead of the stage */
+}
+
+#endif /* COGL_HAS_X11_SUPPORT */
